<?php

namespace Drupal\remote_file_importer;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\remote_file_importer\Entity\DataSource;

/**
 * Manages Batch functionalities for import.
 */
class ImportBatch {

  /**
   * Process the batch.
   *
   * @param string $dataSourcePluginId
   *   Id of f data source plugin.
   * @param array $items
   *   Array of items to be imported.
   *   Each item is an array with keys: local_file, remote_file.
   * @param array $context
   *   Batch context.
   */
  public static function process($dataSourcePluginId, $items, &$context) {
    $pluginManager = \Drupal::service('plugin.manager.remote_file_importer.data_source');
    $dataSource = \Drupal::entityTypeManager()->getStorage('rfi_data_source')->load($dataSourcePluginId);
    $dataSourcePlugin = $pluginManager->createInstance($dataSource->plugin_id);

    if ($dataSourcePlugin->establishConnection($dataSource->settings)) {
      foreach ($items as $item) {
        $isImportSuccessful = $dataSourcePlugin->importFile($item['local_file'], $item['remote_file']);
        if ($dataSource->keep_modify) {
          static::importModifiedDate($item, $dataSource, $dataSourcePlugin);
        }
        if ($dataSource->delete_after_import && $isImportSuccessful) {
          // Delete the remote file after import.
          $dataSourcePlugin->deleteRemoteFile($item['remote_file']);
        }

        $context['results'][] = $item['local_file'];
      }
      $dataSourcePlugin->closeConnection();
    }
  }

  /**
   * Import the modified date of the file.
   *
   * @param array $item
   *   The item to import.
   * @param \Drupal\remote_file_importer\Entity\DataSource $dataSource
   *   The data source entity.
   * @param \Drupal\remote_file_importer\Plugin\DataSourceInterface $dataSourcePlugin
   *   The data source plugin instance.
   */
  public static function importModifiedDate($item, DataSource $dataSource, $dataSourcePlugin) {
    if (!isset($item['metadata']['modified_date']) || empty($item['metadata']['modified_date'])) {
      \Drupal::messenger()
        ->addMessage(t('Missing or invalid modified date for the file @file.', ['@file' => $item['local_file']]));
      return;
    }
    // Sets source timezone.
    if (!$dataSource->source_timezone || !in_array($dataSource->source_timezone, \DateTimeZone::listIdentifiers())) {
      \Drupal::messenger()
        ->addMessage(t('Invalid source timezone'));
      return;
    }

    // Sets site/server timezone.
    $siteTimezone = \Drupal::config('system.date')->get('timezone.default') ?: date_default_timezone_get();
    // Sets source date format.
    $sourceDateFormat = $dataSource->source_date_format ?: $dataSourcePlugin->getDefaultDateFormat();

    // Create a DrupalDateTime object from the inputs.
    try {
      $modifiedDate = DrupalDateTime::createFromFormat(
        $sourceDateFormat,
        $item['metadata']['modified_date'],
        new \DateTimeZone($dataSource->source_timezone)
      );
  
      // Check if parsing was successful.
      if ($modifiedDate instanceof DrupalDateTime && !$modifiedDate->hasErrors()) {
        // Convert to site/server timezone.
        $modifiedDate->setTimezone(new \DateTimeZone($siteTimezone));
  
        // Get timestamp and apply to file.
        $timestamp = $modifiedDate->getTimestamp();
        touch($item['local_file'], $timestamp);
      }
      else {
        \Drupal::messenger()
          ->addMessage(t('Error parsing modified date for the file @file.', ['@file' => $item['local_file']]));
      }
    }
    catch (\Exception $e) {
      \Drupal::messenger()
        ->addMessage(t('Error creating date object: @message', ['@message' => $e->getMessage()]));
      return;
    }
    
  }

  /**
   * Clean up remote deleted files.
   *
   * @param string $source_folder
   *   The datasource destination folder.
   * @param array $files
   *   Array of files to be deleted.
   * @param array $context
   *   Batch context.
   */
  public static function cleanUp(string $source_folder, $files, &$context) {
    $fileSystem = \Drupal::service('file_system');
    foreach ($files as $file) {
      // Remove files that are not in remote folder.
      $file = $fileSystem->delete($file);
    }

    // Remove empty folders.
    $cleanUpService = \Drupal::service('remote_file_importer.cleanup_service');
    $cleanUpService->deleteEmptySubfolders($fileSystem->realpath($source_folder));
  }

  /**
   * Called when the batch is finished.
   *
   * @param bool $success
   *   Whether the batch finished successfully.
   * @param array $results
   *   Array of results.
   * @param array $operations
   *   Array of operations.
   * @param float $elapsed
   *   Time elapsed.
   */
  public static function finished($success, $results, $operations, $elapsed) {
    if ($success) {
      $message = \Drupal::translation()
        ->formatPlural(count($results), 'One file imported.', '@count files imported.');
    }
    else {
      $message = t('Finished with an error.');
    }
    \Drupal::messenger()
      ->addMessage($message);
  }

}
